#include "./CriticalSection.h"

CriticalSection::CriticalSection(void)
{
#if defined(_MSC_VER) || defined(__BORLANDC__)
#if defined(WINAPI_FAMILY) && WINAPI_FAMILY==WINAPI_FAMILY_PHONE_APP
    InitializeCriticalSectionEx(&m_CriticalSection, 0, 0);
#else
    InitializeCriticalSection(&m_CriticalSection);
#endif
#endif

#if defined(_POSIX_THREADS)
	pthread_mutexattr_t attrs;

	pthread_mutexattr_init(&attrs);
	pthread_mutexattr_settype(&attrs, PTHREAD_MUTEX_RECURSIVE);

	pthread_mutex_init(&m_MutexId, &attrs);
#endif

#if defined(__FREERTOS__)
	//vSemaphoreCreateBinary(m_Semaphore);
	m_Semaphore = xSemaphoreCreateRecursiveMutex();
#endif
}

//-------------------------------------------------------------------------------------------------

CriticalSection::~CriticalSection(void)
{
#if defined(_MSC_VER)
	DeleteCriticalSection(&m_CriticalSection);
#endif

#if defined(_POSIX_THREADS)
	pthread_mutex_destroy(&m_MutexId);
#endif

#if defined(__FREERTOS__)
	vQueueDelete(m_Semaphore);
#endif
}

//-------------------------------------------------------------------------------------------------

void CriticalSection::Enter(void)
{
#if defined(_MSC_VER)
	EnterCriticalSection((const LPCRITICAL_SECTION)&m_CriticalSection);
#endif

#if defined(_POSIX_THREADS)
	pthread_mutex_lock(&m_MutexId);
#endif

#if defined(__FREERTOS__)
	//if (xTaskGetSchedulerState() == taskSCHEDULER_RUNNING)
	if (xTaskGetSchedulerState() != taskSCHEDULER_NOT_STARTED)
	{
		xSemaphoreTakeRecursive(m_Semaphore, portMAX_DELAY);
	}
#endif
}

void CriticalSection::Leave(void)
{
#if defined(_MSC_VER)
	LeaveCriticalSection((const LPCRITICAL_SECTION)&m_CriticalSection);
#endif

#if defined(_POSIX_THREADS)
	pthread_mutex_unlock(&m_MutexId);
#endif

#if defined(__FREERTOS__)
	//if (xTaskGetSchedulerState() == taskSCHEDULER_RUNNING)
	if (xTaskGetSchedulerState() != taskSCHEDULER_NOT_STARTED)
	{
		xSemaphoreGiveRecursive(m_Semaphore);
	}
#endif
}
