#ifndef __IClientEndpoint_h
#define __IClientEndpoint_h

#include "./NetworkingHelpers.h"
/*!
 * \brief Endpoint client interface
 * \details Interface for classes using like client endpoint
 */
class IClientEndpoint
{
	public:
        /*!
         * \brief Destructor
         * \details In this function shoult be called all releases.
         */
        virtual ~IClientEndpoint(void) { }

        /*!
         * \brief Disconnect
         * \details Disconnect from socket. This function must be overriden.
         */
        virtual void Disconnect(void) = 0;

        /*!
         * \brief Match
         * \param[in] ipv4 Compared IP Address.
         * \param[in] port Compared IP Port.
         * \return
         * \li \c true - IP port and address maches the current value
         * \li \c false - IP port or address don't maches the current value
         * \details Compare required IP address and port with current settings. This function must be overriden
         */
        virtual bool Match(uint32b ipv4, uint16b port) = 0;
        /*!
         * \brief WantRWAction
         * \param[out] wantRead Client will read (true)
         * \param[out] wantWrite Client will write (true)
         * \details Check communication read/write request. This function must be overriden
         */
        virtual void WantRWAction(bool *wantRead, bool *wantWrite) = 0;
        /*!
         * \brief Get socket
         * \return Current socket
         * \details Function for get current communication socket
         */
        virtual SharkSocket GetSocket(void) = 0;

        /*!
         * \brief Read event
         * \return Data reading status
         * \li \c true - data read
         * \li \c false - data not read
         * \details Ready to read from socket. If client want read, call this function. This function must be overriden
         */
        virtual bool OnReadEvent(void) = 0;
        /*!
         * \brief Write event
         * \return Data writing status
         * \li \c true - data written
         * \li \c false - data not written
         * \details Request for writing data to socket. If client want write, call this function. This function must be overriden
         */
        virtual bool OnWriteEvent(void) = 0;
        /*!
         * \brief Exception event
         * \details Exception raised. Make corresponding actions
         */
        virtual void OnExceptionEvent(void) = 0;

        /*!
         * \brief Check timeout
         * \details Request for check timeout. This function holds action after timeout reached. This function must be overriden
         */
        virtual void CheckTimeout(void) = 0;

        /*!
         * \brief Is connected
         * \return
         * \li \c true - connected
         * \li \c false - diconnected
         * \details Endpoint connection status
         */
        virtual bool IsConnected(void) { return (true); }
    
        /*!
         * \brief Check one read cycle passed
         * \return
         * \details
         * \todo Doplnit popisek. Co by se tu mělo dít
         */
        virtual bool CheckOneReadCyclePassed(void) { return (true); }
        /*!
         * \brief Enable writing cycles
         * \details
         * \todo Doplnit popisek. Co by se tu mělo dít
         */
        virtual void EnableWritingCycles(void) { }

		// pokud je request false, jde o upozorneni ze je treba skoncit, konec nastane tehdy, kdyz vsechny endpointy vrati true
        /*!
         * \brief Shutdown request
         * \param[in] request Flag for terminate
         * \return ??
         * \details Request for shutting down socket.
         * \todo Není mi jasná návratová hodnota. Dále doplnit rozdíl mezi Disconnect a ShutdownRequest. Doplnit.
         */
        virtual bool ShutdownRequest(bool request)
		{ 
			return (request);
		}
};

#endif
