#ifndef __ModbusCrc_h
#define __ModbusCrc_h

#include "./datatypes.h"

/*!
 * \brief Calculate CRC
 * \details Static class for calculating ModBus CRC16 checksum
 */
class ModbusCrc
{
private:
    /*!
     * \brief Initialize table (high bytes)
     */
	static uint8b m_CRCHi[];
    /*!
     * \brief Initialize table (lo bytes)
     */
    static uint8b m_CRCLo[];
public:
    /*!
     * \brief CRC16
     * \param[in] puchMsg Array for calculating CRC
     * \param[in] usDataLen Array length
     * \return Calculated checksum
     * \details Calculate simple ModBus CRC16 from input variables.
     */
	static uint16b CRC16(uint8b *puchMsg, uint32b usDataLen);
    /*!
     * \brief Streaming CRC16 first
     * \param[in] puchMsg Array for calculating CRC
     * \param[in] usDataLen Array length
     * \return Calculated partial checksum
     * \details Calculate simple ModBus CRC16 from input variables (first circle). This function is used with StreamingCRC16Next for long data array.
     */
    static uint16b StreamingCRC16First(uint8b *puchMsg, uint32b usDataLen);
    /*!
     * \brief Streaming CRC16 first
     * \param[in] puchMsg Array for calculating CRC
     * \param[in] usDataLen Array length
     * \param[in] prevCrc Previous calculated CRC from StreamingCRC16First or StreamingCRC16Next function
     * \return Calculated partial checksum
     * \details Cooperate with StreamingCRC16First calculate long data array. This function can be called recursive for long data array.
     */
    static uint16b StreamingCRC16Next(uint8b *puchMsg, uint32b usDataLen, uint16b prevCrc);
};

#endif
