/*!
 * \file SSCPProtocolErrors.h
 *
 * Define communication errors
 */
#ifndef __SSCPProtocolErrors_h
#define __SSCPProtocolErrors_h

/*!
 * \brief Namespace for SscpError enum
 */
namespace SscpError
{
    /*!
     * \brief Communication errors
     * \details Communication errors codes
     * \enum SscpError
     */
	typedef enum
	{
        /*!
         * \brief 0
         * \details No error - communication success
         */
		NoError = 0,

        /*!
         * \brief 1
         * \details _Not sent by controller, used internally_
         */
        NoResponse = 1,
        /*!
         * \brief 2
         * \details _Not sent by controller, used internally_
         */
        FailedToConnect = 2,
        /*!
         * \brief 3
         * \details _Not sent by controller, used internally_
         */
        NotImplemented = 3,
        /*!
         * \brief 4
         * \details _Not sent by controller, used internally_
         */
        InvalidFunctionReceived = 4,

        /*!
         * \brief 0xffff
         * \details Insufficient rights
         */
        InsufficientRights = 0xffff,
        /*!
         * \brief 0xfffe
         * \details Unknown function
         */
        UnknownFunction = 0xfffe,
        /*!
         * \brief 0xfffd
         * \details Invalid version
         */
        InvalidVersion = 0xfffd,
        /*!
         * \brief 0xfffc
         * \details Device not initialized
         */
        DeviceNotInitialized = 0xfffc,

        /*!
        * \brief 0x0101
        * \details WrongLogin
        */
        WrongLogin = 0x0101,

        /*!
         * \brief 0x0102
         * \details NoSuchFile
         */
        NoSuchFile = 0x102,
        /*!
         * \brief 0x0103
         * \details NoSuchVariable
         */
        NoSuchVariable = 0x103,
        /*!
         * \brief 0x0104
         * \details NoSuchTask
         */
        NoSuchTask = 0x104,

        /*!
         * \brief 0x0105
         * \details WrongOrder
         */
        WrongOrder = 0x105,
        /*!
         * \brief 0x0106
         * \details WrongParameter
         */
        WrongParameter = 0x106,
        /*!
         * \brief 0x0107
         * \details InvalidGroupId
         */
        InvalidGroupId = 0x107,
        /*!
         * \brief 0x0108
         * \details TransmissionInProgress
         */
        TransmissionInProgress = 0x108,
        /*!
         * \brief 0x0109
         * \details NotRegistered
         */
        NotRegistered = 0x109,

        /*!
         * \brief 0x010a
         * \details WriteFailed
         */
        WriteFailed = 0x10a,
        /*!
         * \brief 0x010b
         * \details NotAllDataReceived
         */
        NotAllDataReceived = 0x10b,
        /*!
         * \brief 0x010c
         * \details InvalidCrc
         */
        InvalidCrc = 0x10c,

        /*!
         * \brief 0x010d
         * \details DataTooLong
         */
        DataTooLong = 0x10d,
        /*!
         * \brief 0x010e
         * \details TooLongUseFileTransfer
         */
        TooLongUseFileTransfer = 0x10e,
        /*!
         * \brief 0x010f
         * \details FileNameTooLong
         */
        FileNameTooLong = 0x10f,
        /*!
         * \brief 0x0110
         * \details VariableCountLimitExceed
         */
        VariableCountLimitExceed = 0x110,
        /*!
         * \brief 0x0111
         * \details OutOfBounds
         */
        OutOfBounds = 0x111,

        /*!
         * \brief 0x0801
         * \details SysCmd_NewImage
         */
        SysCmd_NewImage = 0x801,
        /*!
         * \brief 0x0802
         * \details SysCmd_InvalidImageArea
         */
        SysCmd_InvalidImageArea = 0x802,
        /*!
         * \brief 0x0803
         * \details SysCmd_CreateBootImage
         */
        SysCmd_CreateBootImage = 0x803,
        /*!
         * \brief 0x0804
         * \details SysCmd_WarmReboot
         */
        SysCmd_WarmReboot = 0x804,
        /*!
         * \brief 0x0805
         * \details SysCmd_ColdReboot
         */
        SysCmd_ColdReboot = 0x805,
        /*!
         * \brief 0x0806
         * \details SysCmd_StartPlc
         */
        SysCmd_StartPlc = 0x806,
        /*!
         * \brief 0x0807
         * \details SysCmd_StopPlc
         */
        SysCmd_StopPlc = 0x807,
        /*!
         * \brief 0x0808
         * \details SysCmd_SetMacAddress
         */
        SysCmd_SetMacAddress = 0x808,
        /*!
         * \brief 0x0809
         * \details SysCmd_Timeout
         */
        SysCmd_Timeout = 0x809,
        /*!
         * \brief 0x080c
         * \details SysCmdRequestActive
         */
        SysCmdRequestActive = 0x80c,
        /*!
         * \brief 0x080d
         * \details SysCmdWaitTimeout
         */
        SysCmdWaitTimeout = 0x80d,

        /*!
         * \brief 0x080a
         * \details AlreaydRunning - never sent at the moment
         */
        AlreadyRunning = 0x80a, // never send at the moment
        /*!
         * \brief 0x080b
         * \details AlreadyStopped - never sent at the moment
         */
        AlreadyStopped = 0x80b, // never send at the moment
        /*!
         * \brief 0x0115
         * \details InvalidState
         */
        InvalidState = 0x115,

        /*!
         * \brief 0x0112
         * \details SizeMismatch
         */
        SizeMismatch = 0x112,
        /*!
         * \brief 0x0113
         * \details OperationDenied
         */
        OperationDenied = 0x113,
        /*!
         * \brief 0x0114
         * \details NotLogged
         */
        NotLogged = 0x114,

        /*!
         * \brief 0x0116
         * \details UnknownChannel
         */
        UnknownChannel = 0x116,
        /*!
         * \brief 0x0117
         * \details DriverCommandTimeout
         */
        DriverCommandTimeout = 0x117,
        /*!
         * \brief 0x0118
         * \details UnknownDriverCommand
         */
        UnknownDriverCommand = 0x118,

        /*!
         * \brief 0x0119
         * \details NoResourcesAvailable
         */
        NoResourcesAvailable = 0x119,

        /*!
         * \brief 0xfff
         * \details Unknown error
         */
        Unknown = 0xfff,
    } SscpError;
}

#if defined(SSCPCLIENTLIB_LIBRARY) && defined(EXPORT_LIBRARY)
__declspec(dllexport)
#elif defined(EXPORT_LIBRARY)
__declspec(dllimport)
#endif
/*!
 * \brief GetSscp error text
 * \param[in] error Error ID
 * \return Error text
 * \details Static function for conversion error code to text
 * \sa SscpError
 */
extern const char *GetSscpErrorText(SscpError::SscpError error);

#endif
