#include "./ModbusCrc.h"

// SharkEvaluator.h je tu kvuli StacksSnapshot
// v zadnem pripade ho nepouzivat primo, ale pouze pres IEngineAccess
#if defined(SSCP_ENABLE_SERVER)
#include "./../Engine/SharkEvaluator.h"
#endif

#if defined(_MSC_VER) || defined(__BORLANDC__)
#include <string.h>
#endif

#if defined(__POSIX__)
#if defined(__PIKEOS__)
extern "C" 
{
#include <vm.h>
}
#else
#include <unistd.h>
#if !defined(__OS_X__)
#include <sys/reboot.h>
#endif
#endif
#endif

#include "./SscpProtocolParser.h"

//-------------------------------------------------------------------------------------------------

#if defined(SSCP_ENABLE_CLIENT)
SscpError::SscpError SscpProtocolParser::Login_Confirmation(CommBuffer & buffer, uint8b *protocolVersion, uint16b *maxDataSize, 
	ConnectionRights *rights, uint8b *imageGuid)
{
	SscpError::SscpError errorCode = CheckFunctionAndErrorStats(buffer, SscpFunction::Login, NULL);
	if (errorCode == SscpError::NoError)
	{
		if (protocolVersion)
			*protocolVersion = buffer[buffer.DataOffset + 4];
		if (maxDataSize)
			*maxDataSize = CommBuffer::GetUint16FromBuffer(buffer.Buffer, buffer.DataOffset + 5);
		if (rights)
			*rights = (ConnectionRights)buffer.Buffer[buffer.DataOffset + 7];
		if (imageGuid)
		{
			for(uint8b i = 0; i < 16; i++) // fixme: 16
				imageGuid[i] = buffer[buffer.DataOffset + 8 + i];
		}

		return (SscpError::NoError);
	}
	return (errorCode);
}

//-------------------------------------------------------------------------------------------------

SscpError::SscpError SscpProtocolParser::ReadVariablesDirectlyVars_Confirmation(CommBuffer & buffer, uint8b **valuesPtr, uint32b *valuesLength, uint64b *errorFlags)
{
	uint32b dataLength;
	SscpError::SscpError errorCode = CheckFunctionAndErrorStats(buffer, SscpFunction::ReadVariablesDirectly, &dataLength, errorFlags);
	if (errorCode == SscpError::NoError)
	{
		if (valuesPtr)
			*valuesPtr = buffer.Buffer + buffer.DataOffset + 4;
		if (valuesLength)
			*valuesLength = dataLength;
		return (SscpError::NoError);
	}
	return (errorCode);
}

//-------------------------------------------------------------------------------------------------

SscpError::SscpError SscpProtocolParser::ReadVariableGroupDirectly_Confirmation(CommBuffer & buffer, uint8b **valuesPtr, uint32b *valuesLength, uint64b *errorFlags)
{
	uint32b dataLength;
	SscpError::SscpError errorCode = CheckFunctionAndErrorStats(buffer, SscpFunction::ReadVariableGroupDirectly, &dataLength, errorFlags);
	if (errorCode == SscpError::NoError)
	{
		if (valuesPtr)
			*valuesPtr = buffer.Buffer + buffer.DataOffset + 4;
		if (valuesLength)
			*valuesLength = dataLength;
		return (SscpError::NoError);
	}
	return (errorCode);
}

//-------------------------------------------------------------------------------------------------

SscpError::SscpError SscpProtocolParser::WriteVariableDirectly_Confirmation(CommBuffer & buffer, uint64b *errorFlags)
{
	uint32b dataLength;
	SscpError::SscpError errorCode = CheckFunctionAndErrorStats(buffer, SscpFunction::WriteVariableDirectly, &dataLength, errorFlags);
	return (errorCode);
}

//-------------------------------------------------------------------------------------------------

SscpError::SscpError SscpProtocolParser::WriteVariableGroupDirectly_Confirmation(CommBuffer & buffer, uint64b *errorFlags)
{
	uint32b dataLength;
	SscpError::SscpError errorCode = CheckFunctionAndErrorStats(buffer, SscpFunction::WriteVariableGroupDirectly, &dataLength, errorFlags);
	return (errorCode);
}

//-------------------------------------------------------------------------------------------------

SscpError::SscpError SscpProtocolParser::ReadVariableStructure_Confirmation(CommBuffer & buffer, uint8b **structure, uint32b *length)
{
	uint32b dataLength;
	SscpError::SscpError errorCode = CheckFunctionAndErrorStats(buffer, SscpFunction::ReadVariableStructure, &dataLength);
	if (errorCode == SscpError::NoError)
	{
		if (structure)
			*structure = buffer.Buffer + buffer.DataOffset + 4;
		if (length)
			*length = dataLength;
		return (SscpError::NoError);
	}
	return (errorCode);
}

//-------------------------------------------------------------------------------------------------

SscpError::SscpError SscpProtocolParser::InitiateReceiveData_Confirmation(CommBuffer & buffer, DataTransferProperties *dataProperties)
{
	uint32b dataLength;
	SscpError::SscpError errorCode = CheckFunctionAndErrorStats(buffer, SscpFunction::InitiateDataReceive, &dataLength);
	if (errorCode == SscpError::NoError)
	{
		if (dataProperties)
		{
			buffer.DataOffset += 4;
			dataProperties->Size = CommBuffer::GetUint32FromBuffer(buffer.Buffer, buffer.DataOffset); 
			buffer.DataOffset += 4;
			
			dataProperties->Timestamp = CommBuffer::GetUint64FromBuffer(buffer.Buffer, buffer.DataOffset); 
			buffer.DataOffset += 8;
			
			dataProperties->Crc = CommBuffer::GetUint16FromBuffer(buffer.Buffer, buffer.DataOffset); 
			buffer.DataOffset += 2;
			
			dataProperties->CalculatedCrc = 0xffff;
			dataProperties->FileOffset = 0;
		}
		return (SscpError::NoError);
	}
	return (errorCode);
}

//-------------------------------------------------------------------------------------------------

SscpError::SscpError SscpProtocolParser::ReceiveDataChunk_Confirmation(CommBuffer & buffer, uint32b *dataOffset, uint8b *dest, 
	bool *transferTerminated, DataTransferProperties *dataProperties)
{
	uint32b dataLength;
	SscpError::SscpError errorCode = CheckFunctionAndErrorStats(buffer, SscpFunction::ReceiveDataChunk, &dataLength);
	if (errorCode == SscpError::NoError)
	{
		if (dataLength >= 4)
		{
			dataLength -= 4;
			buffer.DataOffset += 4;

			*dataOffset = CommBuffer::GetUint32FromBuffer(buffer.Buffer, buffer.DataOffset);
			buffer.DataOffset += 4;

			memcpy(dest + (*dataOffset), buffer.Buffer + buffer.DataOffset, dataLength);

			if (dataProperties)
			{
				dataProperties->CalculatedCrc = ModbusCrc::StreamingCRC16Next(buffer.Buffer + buffer.DataOffset, dataLength, dataProperties->CalculatedCrc);
				dataProperties->LastChunkSize = dataLength;
			}

			*dataOffset += dataLength;
			*transferTerminated = dataProperties ? (*dataOffset == dataProperties->Size) : true; // pokud nepredame dataProperties tak koncime po 1. segmentu

			return (SscpError::NoError);
		}
		else
		{
			// fixme: chyba
			*transferTerminated = true;	
			return (SscpError::Unknown);
		}
	}
	return (errorCode);
}

//-------------------------------------------------------------------------------------------------

#if !defined(SSCP_STANDALONE_CLIENT)
SscpError::SscpError SscpProtocolParser::ReceiveDataChunk_Confirmation(CommBuffer & buffer, uint32b *dataOffset, IMemoryAllocator *ma, sint32b allocId,
	bool *transferTerminated, DataTransferProperties *dataProperties)
{
	uint32b dataLength;
	SscpError::SscpError errorCode = CheckFunctionAndErrorStats(buffer, SscpFunction::ReceiveDataChunk, &dataLength);
	if (errorCode == SscpError::NoError)
	{
		if (dataLength >= 4)
		{
			dataLength -= 4;
			buffer.DataOffset += 4;

			*dataOffset = CommBuffer::GetUint32FromBuffer(buffer.Buffer, buffer.DataOffset);
			buffer.DataOffset += 4;

			//memcpy(dest + (*dataOffset), buffer.Buffer + buffer.DataOffset, dataLength);
			if (allocId != -1)
			{
				ma->Write(allocId, *dataOffset, buffer.Buffer + buffer.DataOffset, dataLength);
			}

			if (dataProperties)
			{
				dataProperties->CalculatedCrc = ModbusCrc::StreamingCRC16Next(buffer.Buffer + buffer.DataOffset, dataLength, dataProperties->CalculatedCrc);
				dataProperties->LastChunkSize = dataLength;
			}

			*dataOffset += dataLength;
			*transferTerminated = dataProperties ? (*dataOffset == dataProperties->Size) : true; // pokud nepredame dataProperties tak koncime po 1. segmentu

			return (SscpError::NoError);
		}
		else
		{
			// fixme: chyba
			*transferTerminated = true;	
			return (SscpError::Unknown);
		}
	}
	return (errorCode);
}
#endif

//-------------------------------------------------------------------------------------------------

SscpError::SscpError SscpProtocolParser::InitiateSendData_Confirmation(CommBuffer & buffer)
{
	uint32b dataLength;
	SscpError::SscpError errorCode = CheckFunctionAndErrorStats(buffer, SscpFunction::InitiateDataSend, &dataLength);
	return (errorCode);
}

//-------------------------------------------------------------------------------------------------

SscpError::SscpError SscpProtocolParser::SendDataChunk_Confirmation(CommBuffer & buffer, uint32b *confirmedOffset)
{
	uint32b dataLength;
	SscpError::SscpError errorCode = CheckFunctionAndErrorStats(buffer, SscpFunction::SendDataChunk, &dataLength);
	if (errorCode == SscpError::NoError)
	{
		buffer.DataOffset += 4;
		if (confirmedOffset)
			*confirmedOffset = CommBuffer::GetUint32FromBuffer(buffer.Buffer, buffer.DataOffset);
		return (SscpError::NoError);
	}
	return (errorCode);
}

//-------------------------------------------------------------------------------------------------

SscpError::SscpError SscpProtocolParser::FinishDataSend_Confirmation(CommBuffer & buffer)
{
	uint32b dataLength;
	SscpError::SscpError errorCode = CheckFunctionAndErrorStats(buffer, SscpFunction::FinishDataSend, &dataLength);
	return (errorCode);
}

//-------------------------------------------------------------------------------------------------

SscpError::SscpError SscpProtocolParser::TimeSetup_Confirmation(CommBuffer & buffer, uint64b *timeReturned)
{
	uint32b dataLength;
	SscpError::SscpError errorCode = CheckFunctionAndErrorStats(buffer, SscpFunction::TimeSetup, &dataLength);
	if (errorCode == SscpError::NoError)
	{
		buffer.DataOffset += 4;
		if ((dataLength > 0) && (timeReturned))
			*timeReturned = CommBuffer::GetUint64FromBuffer(buffer.Buffer, buffer.DataOffset);
	}
	return (errorCode);
}

SscpError::SscpError SscpProtocolParser::TimeSetupEx_Confirmation(CommBuffer & buffer, uint64b *timeReturned)
{
    uint32b dataLength;
    SscpError::SscpError errorCode = CheckFunctionAndErrorStats(buffer, SscpFunction::TimeSetupEx, &dataLength);
    if (errorCode == SscpError::NoError)
    {
        buffer.DataOffset += 4;
        if ((dataLength > 0) && (timeReturned))
            *timeReturned = CommBuffer::GetUint64FromBuffer(buffer.Buffer, buffer.DataOffset);
    }
    return (errorCode);
}

//-------------------------------------------------------------------------------------------------
#endif /* SSCP_ENABLE_CLIENT */
