#include "./ModbusCrc.h"

// SharkEvaluator.h je tu kvuli StacksSnapshot
// v zadnem pripade ho nepouzivat primo, ale pouze pres IEngineAccess
#if defined(SSCP_ENABLE_SERVER)
#include "./../Engine/SharkEvaluator.h"
#endif

#if defined(_MSC_VER) || defined(__BORLANDC__)
#include <string.h>
#endif

#if defined(__POSIX__)
#if defined(__PIKEOS__)
extern "C" 
{
#include <vm.h>
}
#else
#include <unistd.h>
#if !defined(__OS_X__)
#include <sys/reboot.h>
#endif
#endif
#endif

#include "./SscpProtocolParser.h"

//-------------------------------------------------------------------------------------------------

#if defined(SSCP_ENABLE_CLIENT)
void SscpProtocolParser::Login_Request(CommBuffer & buffer, const char *username, const char *password, const char *proxyId)
{
	// spolecna cast
	uint8b nameLength = (username) ? strlen(username) : 0;
	uint8b passLength = (password) ? 16 : 0; // md5 fixed
	uint8b proxyIdLength = proxyId ? strlen(proxyId) : 0;

	uint16b dataLen = 5 + nameLength + passLength;
	if (proxyId)
		dataLen += 1 + proxyIdLength;
	buffer.DataOffset = ComposeHeader(SscpFunction::Login, dataLen, buffer);
	buffer.DataOffset += sizeof(SSCPProtocolHeader);

	// vlastni login
	buffer[buffer.DataOffset++] = proxyId ? 2 : 1; // requested version
	buffer.DataOffset = CommBuffer::SetUint16ToBuffer(buffer.Buffer, buffer.DataOffset, buffer.BufferSize - 40); // fixme: 40
	
	buffer[buffer.DataOffset++] = nameLength;
	for (uint8b i = 0; i < nameLength; i++)
		buffer[buffer.DataOffset++] = username[i];

	buffer[buffer.DataOffset++] = passLength;
	for (uint8b i = 0; i < passLength; i++)
		buffer[buffer.DataOffset++] = password[i];

	if (proxyId)
	{
		buffer[buffer.DataOffset++] = proxyIdLength;
		for (uint8b i = 0; i < proxyIdLength; i++)
			buffer[buffer.DataOffset++] = proxyId[i];
	}
}

//-------------------------------------------------------------------------------------------------

void SscpProtocolParser::ReadVariablesDirectlyHeader_Request(CommBuffer & buffer, bool useOnlyCommUid, bool useVmUid, uint32b varsCount)
{
	uint32b dataLen = 1 + varsCount * (useOnlyCommUid ? sizeof(uint32b) : (3 * sizeof(uint32b)));
	buffer.DataOffset = ComposeHeader(SscpFunction::ReadVariablesDirectly, dataLen, buffer);
	buffer.DataOffset += sizeof(SSCPProtocolHeader);

	VarAccessFlags::VarAccessFlags flags = 
		(VarAccessFlags::VarAccessFlags)((useOnlyCommUid ? 0 : VarAccessFlags::UseOffsetLength) |
		(useVmUid ? VarAccessFlags::UseVmUid : 0));
	flags = (VarAccessFlags::VarAccessFlags)(flags | VarAccessFlags::GetAllValues);

	buffer[buffer.DataOffset++] = (uint8b)flags;
}

//-------------------------------------------------------------------------------------------------

void SscpProtocolParser::RegisterGroupHeader_Request(CommBuffer & buffer, uint8b groupId, bool useOnlyCommUid, bool useVmUid, uint32b varsCount)
{
	uint32b dataLen = 2 + varsCount * (useOnlyCommUid ? sizeof(uint32b) : (3 * sizeof(uint32b)));
	buffer.DataOffset = ComposeHeader(SscpFunction::RegisterVariableGroup, dataLen, buffer);
	buffer.DataOffset += sizeof(SSCPProtocolHeader);

	VarAccessFlags::VarAccessFlags flags = 
		(VarAccessFlags::VarAccessFlags)((useOnlyCommUid ? 0 : VarAccessFlags::UseOffsetLength) |
		(useVmUid ? VarAccessFlags::UseVmUid : 0));

	buffer[buffer.DataOffset++] = groupId;
	buffer[buffer.DataOffset++] = (uint8b)flags;
}

//-------------------------------------------------------------------------------------------------

void SscpProtocolParser::AppendVariable_Request(CommBuffer & buffer, uint32b commUid, uint32b offset, uint32b length)
{
	buffer.DataOffset = CommBuffer::SetUint32ToBuffer(buffer.Buffer, buffer.DataOffset, commUid);
	buffer.DataOffset = CommBuffer::SetUint32ToBuffer(buffer.Buffer, buffer.DataOffset, offset);
	buffer.DataOffset = CommBuffer::SetUint32ToBuffer(buffer.Buffer, buffer.DataOffset, length);
}

void SscpProtocolParser::AppendVariable_Request(CommBuffer & buffer, uint32b commUid)
{
	buffer.DataOffset = CommBuffer::SetUint32ToBuffer(buffer.Buffer, buffer.DataOffset, commUid);
}

//-------------------------------------------------------------------------------------------------

void SscpProtocolParser::ReadVariableGroupDirectly_Request(CommBuffer & buffer, uint8b groupId, VarAccessFlags::VarAccessFlags flags)
{
	uint32b dataLen = 2;
	buffer.DataOffset = ComposeHeader(SscpFunction::ReadVariableGroupDirectly, dataLen, buffer);
	buffer.DataOffset += sizeof(SSCPProtocolHeader);

	VarAccessFlags::VarAccessFlags flagsTmp = (VarAccessFlags::VarAccessFlags)(flags & VarAccessFlags::GetFormatMask);

	buffer[buffer.DataOffset++] = groupId;
	buffer[buffer.DataOffset++] = (uint8b)flagsTmp;
}

//-------------------------------------------------------------------------------------------------

void SscpProtocolParser::WriteVariableDirectly_Request(CommBuffer & buffer, bool useOnlyCommUid, bool useVmUid, VarAccessFlags::VarAccessFlags flags, 
	uint32b varCount, uint32b variablesLength, uint32b *totalLength)
{
	uint32b headerSize = useOnlyCommUid ? 4 : 12;
	uint32b dataSize = 1 + ((variablesLength > 0) ? 1 : 0) + varCount * headerSize + variablesLength;

	if (totalLength)
		*totalLength = dataSize;

	buffer.DataOffset = ComposeHeader(SscpFunction::WriteVariableDirectly, dataSize, buffer);
	buffer.DataOffset += sizeof(SSCPProtocolHeader);

	uint8b flagsTmp = useOnlyCommUid ? 0 : VarAccessFlags::UseOffsetLength;
	flagsTmp |= (useVmUid) ? VarAccessFlags::UseVmUid : 0;
	flagsTmp |= (variablesLength == 0) ? VarAccessFlags::WriteLargeData : 0;

	flagsTmp |= flags & VarAccessFlags::GetFormatMask;

	buffer += flagsTmp;
	if (variablesLength > 0)
		buffer += (uint8b)varCount;
}

//-------------------------------------------------------------------------------------------------

void SscpProtocolParser::WriteVariableGroupDirectly_Request(CommBuffer & buffer, uint8b groupId, VarAccessFlags::VarAccessFlags flags, uint32b variableDescCount,
	uint32b variablesLength)
{
	uint8b writeMode = flags & VarAccessFlags::SetFormatMask;

	buffer.DataOffset = ComposeHeader(SscpFunction::WriteVariableGroupDirectly, 2 + variableDescCount + variablesLength +
		((writeMode == VarAccessFlags::SetSelectedValues) ? 1 : 0), buffer);
	buffer.DataOffset += sizeof(SSCPProtocolHeader);

	uint8b flagsTmp = (variablesLength == 0) ? VarAccessFlags::WriteLargeData : 0;
	flagsTmp |= flags & VarAccessFlags::SetFormatMask;

	buffer += groupId;
	buffer += flagsTmp;
	if (writeMode == VarAccessFlags::SetSelectedValues)
		buffer += (uint8b)variableDescCount;
}

//-------------------------------------------------------------------------------------------------

void SscpProtocolParser::ReadVariableStructure_Request(CommBuffer & buffer, uint8b flags, uint32b varCount)
{
	buffer.DataOffset = ComposeHeader(SscpFunction::ReadVariableStructure, (sizeof(SSCPProtocolReadVariableStructureRequest) - sizeof(SSCPProtocolHeader)) + 
		varCount * sizeof(uint32b), buffer);
	buffer.DataOffset += sizeof(SSCPProtocolHeader);

	buffer += flags;
}

//-------------------------------------------------------------------------------------------------

void SscpProtocolParser::InitiateReceiveData_Request(CommBuffer & buffer, const char *filename)
{
	uint8b filenameLen = strlen(filename);
	uint32b dataLen = 1 + filenameLen;
	buffer.DataOffset = ComposeHeader(SscpFunction::InitiateDataReceive, dataLen, buffer);
	buffer.DataOffset += sizeof(SSCPProtocolHeader);

	buffer[buffer.DataOffset++] = filenameLen;
	for (uint8b i = 0; i < filenameLen; i++)
		buffer[buffer.DataOffset++] = filename[i];
}

//-------------------------------------------------------------------------------------------------

void SscpProtocolParser::ReceiveDataChunk_Request(CommBuffer & buffer, uint32b dataOffset)
{
	uint32b dataLen = 4;
	buffer.DataOffset = ComposeHeader(SscpFunction::ReceiveDataChunk, dataLen, buffer);
	buffer.DataOffset += sizeof(SSCPProtocolHeader);

	buffer.DataOffset = CommBuffer::SetUint32ToBuffer(buffer.Buffer, buffer.DataOffset, dataOffset);
}

//-------------------------------------------------------------------------------------------------

void SscpProtocolParser::InitiateSendData_Request(CommBuffer & buffer, const char *filename, DataTransferProperties *dataProperties)
{
	uint8b filenameLen = strlen(filename);
	uint32b dataLen = 13 + filenameLen;
	buffer.DataOffset = ComposeHeader(SscpFunction::InitiateDataSend, dataLen, buffer);
	buffer.DataOffset += sizeof(SSCPProtocolHeader);

	buffer[buffer.DataOffset++] = filenameLen;
	for (uint8b i = 0; i < filenameLen; i++)
		buffer[buffer.DataOffset++] = filename[i];

	buffer.DataOffset = CommBuffer::SetUint32ToBuffer(buffer.Buffer, buffer.DataOffset, dataProperties->Size);
    buffer.DataOffset = CommBuffer::SetUint64ToBuffer(buffer.Buffer, buffer.DataOffset, dataProperties->Timestamp);
}

//-------------------------------------------------------------------------------------------------

void SscpProtocolParser::SendDataChunk_Request(CommBuffer & buffer, uint16b maxFrameSize, uint8b *src, uint32b dataSize, uint32b *offset, bool *transmissionComplete)
{
	uint16b chunkSize = SharkMin((uint16b)(maxFrameSize - 4), (uint16b)(dataSize - (*offset)));
	uint32b dataLen = 4 + chunkSize;
	buffer.DataOffset = ComposeHeader(SscpFunction::SendDataChunk, dataLen, buffer);
	buffer.DataOffset += sizeof(SSCPProtocolHeader);

	buffer.DataOffset = CommBuffer::SetUint32ToBuffer(buffer.Buffer, buffer.DataOffset, *offset);

	memcpy(buffer.Buffer + buffer.DataOffset, src + (*offset), chunkSize);

	*offset += chunkSize;

	*transmissionComplete = dataSize == (*offset);
}

#if !defined(SSCP_STANDALONE_CLIENT)
void SscpProtocolParser::SendDataChunk_Request(CommBuffer & buffer, uint16b maxFrameSize, IMemoryAllocator *ma, sint32b allocId, uint32b dataSize, uint32b *offset, 
	bool *transmissionComplete)
{
	uint16b chunkSize = SharkMin((uint16b)(maxFrameSize - 4), (uint16b)(dataSize - (*offset)));
	uint32b dataLen = 4 + chunkSize;
	buffer.DataOffset = ComposeHeader(SscpFunction::SendDataChunk, dataLen, buffer);
	buffer.DataOffset += sizeof(SSCPProtocolHeader);

	buffer.DataOffset = CommBuffer::SetUint32ToBuffer(buffer.Buffer, buffer.DataOffset, *offset);

	if (ma)
		ma->Read(allocId, *offset, buffer.Buffer + buffer.DataOffset, chunkSize);

	*offset += chunkSize;

	*transmissionComplete = dataSize == (*offset);
}
#endif

//-------------------------------------------------------------------------------------------------

void SscpProtocolParser::FinishDataSend_Request(CommBuffer & buffer, uint16b crc)
{
	uint32b dataLen = 2;
	buffer.DataOffset = ComposeHeader(SscpFunction::FinishDataSend, dataLen, buffer);
	buffer.DataOffset += sizeof(SSCPProtocolHeader);

	buffer.DataOffset = CommBuffer::SetUint16ToBuffer(buffer.Buffer, buffer.DataOffset, crc);
}

//-------------------------------------------------------------------------------------------------

void SscpProtocolParser::TimeSetup_Request(CommBuffer & buffer, uint64b time)
{
	uint32b dataLen = time != 0 ? 8 : 0;
	buffer.DataOffset = ComposeHeader(SscpFunction::TimeSetup, dataLen, buffer);
	buffer.DataOffset += sizeof(SSCPProtocolHeader);

	if (time != 0)
		buffer.DataOffset = CommBuffer::SetUint64ToBuffer(buffer.Buffer, buffer.DataOffset, time);
}

//-------------------------------------------------------------------------------------------------

void SscpProtocolParser::TimeSetupEx_Request(CommBuffer & buffer, uint8b cmd, uint8b flags, uint64b time)
{
    uint32b dataLen = (cmd & 0x10) > 0 ? 10 : 2;
    buffer.DataOffset = ComposeHeader(SscpFunction::TimeSetupEx, dataLen, buffer);
    buffer.DataOffset += sizeof(SSCPProtocolHeader);
    // Command
    buffer += cmd;
    // Flags
    buffer += flags;

    if (dataLen > 2)
        buffer.DataOffset = CommBuffer::SetUint64ToBuffer(buffer.Buffer, buffer.DataOffset, time);
}

//-------------------------------------------------------------------------------------------------

#endif

