#ifndef __TcpClient_h
#define __TcpClient_h

#if !defined(SSCP_STANDALONE_CLIENT)
#include "./../../RT/Communication/ICallbackChannel.h"
#else
#include "./Thread.h"
#endif

#if defined(STANDALONE_SUPPORT)
#include "./IClientEndpoint.h"
#else
#include "./../../RT/Communication/IClientEndpoint.h"
#endif

/*!
 * \class TcpUdpClient
 * \brief Main TCP/UDP client class
 * \details Main TCP/UDP client class for communication with PLC
 */

#if !defined(SSCP_STANDALONE_CLIENT)
class TcpUdpClient : public ICallbackChannel
#else
class TcpUdpClient : public Thread
#endif
{
	private:
        /*!
         * \brief Timeout
         * \details Communication timeout in ms
         */
		uint32b m_Timeout; // v ms
        /*!
         * \brief Endpoints count
         * \details maximum endpoints (clinets)
         */
		uint16b m_EndpointsCount;

        /*!
         * \brief Endpoints array
         * \details Array of PLC clients definitions
         */
		IClientEndpoint * * m_Endpoints;
	public:
        /*!
         * \brief Constructor
         * \param[in] endpointsCount Maximum connected clients
         * \details Initialize class
         */
		TcpUdpClient(uint16b endpointsCount);
        /*!
         * \brief Destructor
         * \details Remove all clients definitions and free accessed memeory
         */
        virtual ~TcpUdpClient(void);

        /*!
         * \brief Set endpoint
         * \param[in] index Endpoint index
         * \param[in] endpoint
         * \details Set endpoint at current position. If \c index is bigger than \c m_EndpointsCount, nothing happened.
         */
		void SetEndpoint(uint32b index, IClientEndpoint * endpoint);
        /*!
         * \brief Find endpoint
         * \param[in] ipv4 IP address
         * \param[in] port IP port
         * \return Endpoint index
         * \details Search all assigned endpoint for selected IP address (\c ipv4) and \c port. Return first find of client.
         */
		sint32b FindEndpoint(uint32b ipv4, uint16b port);
        /*!
         * \brief Clear and free endpoint
         * \details Remove all clients definitions and free accessed memeory
         */
		void ClearAndFreeEndpoint(void);

        /*!
         * \brief Initialize
         * \details Initialization function for object. Made some initialization here.
         */
		virtual void Initialize(void);
#if defined(SSCP_STANDALONE_CLIENT)
        /*!
         * \brief Execute
         * \details Main execution function. This function communicates with PLC and read responds
         */
		virtual void Execute() 
		{ 
			Initialize(); 
			Execute(&m_Running); 
			Deinitialize(); 
		}
#endif
        /*!
         * \brief Execute
         * \param[in] running Running flag for stopping thread
         * \details Main execution function. This function communicates with PLC and read responds.
         */
		virtual void Execute(volatile bool * running);
        /*!
         * \brief Deinitialize
         * \details Stops communication with all connected clients and disconnect them from soket
         */
		virtual void Deinitialize(void);
#if !defined(SSCP_STANDALONE_CLIENT)
        /*!
         * \brief CheckOneReadCyclePassed
         * \return
         * \details Used only without SSCP_STANDALONE_CLIENT definition
         * \todo Doplnit
         */
		virtual bool CheckOneReadCyclePassed(void);
        /*!
         * \brief CheckOneReadCyclePassed
         * \return
         * \details Used only without SSCP_STANDALONE_CLIENT definition
         * \todo Doplnit
         */
        virtual void EnableWritingCycles(void);
#endif
};

#endif
