#if defined(__FREERTOS__)
extern "C" {
/* Kernel includes. */
#include <FreeRTOS.h>
#include "task.h"
#include "queue.h"

#include "./../../HW/FreeRTOS-ARM/STM32F407/LwIP/port/arch/sys_arch.h"
sys_thread_t sys_thread_new(char *name, void (* thread)(void *arg), void *arg, int stacksize, int prio);
}
#undef _MSC_VER
//#define DEFAULT_TASK_PRIORITY		( configMAX_PRIORITIES - 1 )
#endif

#include "./Thread.h"

#if defined(GLOBAL_WDTMANAGER)
#include "./WatchdogManager.h"
#endif

#include "./SystemCalls.h"
#if !defined(SSCP_STANDALONE_CLIENT)
#include "./ThreadPriorities.h"
#include "./SystemLogger.h"
#else
#define POSIX_SCHEDULING_POLICY		SCHED_RR
#define POSIX_SCHEDULING_DEFPRIO	70
#include "./SystemLogger.h"
#endif

#define FAC_THREAD		NULL

#if defined(_MSC_VER) || defined(__BORLANDC__)
#include <process.h>

#if defined(__THREAD_ID__)
const DWORD MS_VC_EXCEPTION=0x406D1388;

#pragma pack(push,8)
typedef struct tagTHREADNAME_INFO
{
   DWORD dwType; // Must be 0x1000.
   LPCSTR szName; // Pointer to name (in user addr space).
   DWORD dwThreadID; // Thread ID (-1=caller thread).
   DWORD dwFlags; // Reserved for future use, must be zero.
} THREADNAME_INFO;
#pragma pack(pop)

void SetThreadNameWin32( DWORD dwThreadID, char* threadName)
{
	THREADNAME_INFO info;
	info.dwType = 0x1000;
	info.szName = threadName;
	info.dwThreadID = dwThreadID;
	info.dwFlags = 0;

	__try
	{
		RaiseException( MS_VC_EXCEPTION, 0, sizeof(info)/sizeof(ULONG_PTR), (ULONG_PTR*)&info );
	}
	__except(EXCEPTION_EXECUTE_HANDLER)
	{
	}
}
#endif

#endif

#ifdef _POSIX_THREADS
#ifndef __OS_X__
#include <sys/prctl.h>
#else
#include <sys/types.h>
#endif
#include <sys/time.h>
#include <sys/resource.h>
#endif
#if defined(_MSC_VER) || defined(__BORLANDC__)
#if WINAPI_FAMILY==WINAPI_FAMILY_PHONE_APP
DWORD WINAPI ThreadMainExecute(LPVOID threadObject)
#else
static unsigned int __stdcall ThreadMainExecute(void * threadObject)
#endif
#endif

#if defined(_POSIX_THREADS)
/*!
 * \brief
 * \details Main execution function. This runs Execute function in thread mode.
 */
static void * ThreadMainExecute(void * threadObject)
#endif
#if defined(__FREERTOS__)
static void ThreadMainExecute(void * threadObject)
#endif
{
#if !defined(SSCP_STANDALONE_CLIENT) && defined(__THREAD_ID__)
	Logger.LogDebug(FAC_THREAD, "Starting thread %s with parameters", ((Thread*)threadObject)->m_ThreadName);
#endif

#if defined(__POSIX__) && !defined(__OS_X__)
	if (strlen(((Thread*)threadObject)->m_ThreadName) > 16)
		((Thread*)threadObject)->m_ThreadName[15] = 0;
	prctl(PR_SET_NAME, (unsigned long)((Thread*)threadObject)->m_ThreadName);
#endif


	((Thread *) threadObject)->Execute();

	((Thread *) threadObject)->OnStopped();

#if !defined(SSCP_STANDALONE_CLIENT) && defined(__THREAD_ID__)
	Logger.LogDebug(FAC_THREAD, "Thread %s stopped", ((Thread*)threadObject)->m_ThreadName);
#endif

#if defined(_MSC_VER) || defined(__BORLANDC__)
#if WINAPI_FAMILY==WINAPI_FAMILY_PHONE_APP
    ExitThread(0);
#else
    _endthread();
#endif
    return 0;
#endif

#if defined(__POSIX__)
	pthread_exit(0);
#endif
#if defined(__FREERTOS__)
	vTaskDelete(NULL);
#endif

}

#if defined(__THREAD_ID__)
void Thread::SetThreadName(const char *threadName)
{
	if (threadName)
		strncpy(m_ThreadName, threadName, MAX_THREAD_ID_LENGTH);
}
#endif

Thread::Thread(void)
{
	m_Running = false;
	m_Stopped = true;

#if defined(__THREAD_ID__)
	m_ThreadName[0] = 0;	
#endif
}

//-------------------------------------------------------------------------------------------------

Thread::~Thread(void)
{
}

//-------------------------------------------------------------------------------------------------

bool Thread::Start(uint32b priority)
{
	// fixme: pouzit parametr priority i jinde nez an freertosu
	bool result = false;

	OnStarted();

#if defined(_MSC_VER) || defined(__BORLANDC__)
#if WINAPI_FAMILY==WINAPI_FAMILY_PHONE_APP
    DWORD tId;
    HANDLE handle = CreateThread(NULL, 0, ThreadMainExecute, this, 0, &tId);
#else
    uint32b tId;
    HANDLE handle = (HANDLE)_beginthreadex(NULL, 0, ThreadMainExecute, this, 0, &tId);
#endif
    result = (int)handle != -1;
#if defined(__THREAD_ID__)
	SetThreadNameWin32(tId, m_ThreadName);
#endif
#endif

#if defined(_POSIX_THREADS)
	pthread_attr_t attr;

	pthread_attr_init(&attr);

	//int policy = SCHED_OTHER;
	int policy = POSIX_SCHEDULING_POLICY;
	struct sched_param param;
	
#ifndef __ANDROID__
    if (policy != SCHED_OTHER)
	{
        if (pthread_attr_setinheritsched(&attr, PTHREAD_INHERIT_SCHED) == 0)
		{
			//pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);
			if (pthread_attr_setschedpolicy(&attr, policy) == 0)
			{
				// vyssi cislo = vyssi priorita
				param.sched_priority = POSIX_SCHEDULING_DEFPRIO + priority;
				if (pthread_attr_setschedparam(&attr, &param) != 0)
					Logger.LogError(FAC_THREAD, "Failed to set priority for thread %s", m_ThreadName);
			}
			else
				Logger.LogError(FAC_THREAD, "Failed to set policy for thread %s", m_ThreadName);
		}
		else
			Logger.LogError(FAC_THREAD, "Failed to set inheritency for thread %s", m_ThreadName);
	}
#endif
	sint32b res = pthread_create(&m_ThreadId, &attr, ThreadMainExecute, this);
	if (res == 0)
	{
#if defined(__THREAD_ID__)
#if 0 // fixme: pro pc linux neexistuje
		pthread_setname_np(m_ThreadId, m_ThreadName);
#endif
#endif
		if (policy == SCHED_OTHER)
		{
			// change nice level
#ifndef __OS_X__
            setpriority(PRIO_PROCESS, m_ThreadId, POSIX_SCHEDULING_DEFPRIO + priority);
#endif
        }

		if (pthread_getschedparam(m_ThreadId, &policy, &param) == 0)
			Logger.LogDebug(FAC_THREAD, "%s - scheduling parameters - policy = %d, priority = %d", m_ThreadName, policy, param.sched_priority);
		else
			Logger.LogError(FAC_THREAD, "get scheduling params failed");

		result = true;
	}
	else
	{
#if defined(__THREAD_ID__)
		Logger.LogError(FAC_THREAD, "Failed to create thread %s (%s)", m_ThreadName, strerror(res));
#else
		Logger.LogError(FAC_THREAD, "Failed to create thread (%s)", strerror(res));
#endif
	}
#endif

#if defined(__FREERTOS__)
	//musi se to hnat pres tuto vrstvu jinak LWIP nefungujou timery
#if defined(__THREAD_ID__)
	result = (sys_thread_new(m_ThreadName, ThreadMainExecute, this, Thread::DEFAULT_STACK_SIZE / sizeof(portSTACK_TYPE), priority) != NULL);
#else
	result = (sys_thread_new(( char * ) "ThreadTask", ThreadMainExecute, this, Thread::DEFAULT_STACK_SIZE / sizeof(portSTACK_TYPE), priority) != NULL);
#endif
#endif

	if (!result)
	{
		// pokud se nepovede pustit vlakno, nastavime ho zpet jako zastavene, aby "spravne" zafungoval join
		m_Running = false;
		m_Stopped = true;
	}

	return result;
}

//-------------------------------------------------------------------------------------------------

void Thread::SetStopSignal(void)
{
	m_Running = false;
}

//-------------------------------------------------------------------------------------------------

bool Thread::Join(uint8b wdtClientId, uint32b wdtPeriod, uint32b timeoutInMs)
{
	// fixme: pouzit nejaka primitiva misto busy waitingu
#if defined(__THREAD_ID__) && !defined(SSCP_STANDALONE_CLIENT)
	Logger.LogDebug(FAC_THREAD, "Waiting for thread '%s' end", m_ThreadName);
#endif

	while ((!m_Stopped)
#if defined(__FREERTOS__)
	 || (xGetDeletedTasksCount() > 0)	//vTaskDelete neuvolni pamet tasku,
	 									//uvolni se az naslednymi pruchody idle tasku,
										//takze tady pockame dokud se neuvolni pamet vsech ukoncovanych tasku
#endif
	)
	{
		SystemCalls::Sleep(100);

#if defined(GLOBAL_WDTMANAGER)
		if (wdtClientId != 0xff)
			WdtManager.Reset(wdtClientId, wdtPeriod);
#endif

		if (timeoutInMs >= 100)
		{
			timeoutInMs -= 100;
			if (timeoutInMs == 0)
			{
#if defined(__THREAD_ID__) && !defined(SSCP_STANDALONE_CLIENT)
				Logger.LogDebug(FAC_THREAD, "Join - Wait for thread '%s' timeout", m_ThreadName);
#endif
				return (false);
			}
		}
	}

#if defined(__THREAD_ID__) && !defined(SSCP_STANDALONE_CLIENT)
	Logger.LogDebug(FAC_THREAD, "Join - Thread '%s' ended", m_ThreadName);
#endif

#if defined(__POSIX__)
    pthread_join(m_ThreadId, NULL);
#endif
	return (true);
}

//-------------------------------------------------------------------------------------------------

void Thread::OnStarted(void)
{
	m_Running = true;
	m_Stopped = false;
}

//-------------------------------------------------------------------------------------------------

void Thread::OnStopped(void)
{
	m_Stopped = true;
}

//-------------------------------------------------------------------------------------------------

